// module noten.cpp
// Copyright Dr.E.Huckert 08-2017

// Compiled under Windows with gcc/g++ (Mingw):works!
// See the make files noten.mak (Windows) and notenU.mak (Unix)

#include <SFML/Audio.hpp>
#include <SFML/Graphics.hpp>

#include <iostream>
#include <inttypes.h>  // not really needed
#include <string.h>

// ---------------------------------------------------
// define five horizontal lines corresponding to a music system
sf::VertexArray makeNotenzeile(int xp1, int xp2, int yp)
{
  #define LDIST 10
  #define LINES 5
  // Horiz.line 1
  sf::VertexArray system(sf::Lines, LINES * 2);
  //
  int m = 0;
  for (int n=0; n < LINES; n++)
  {
    system[m].position   = sf::Vector2f(xp1, yp + (n * LDIST));
    system[m].color      = sf::Color::Black;
    system[m+1].position = sf::Vector2f(xp2, yp + (n * LDIST));
    system[m+1].color    = sf::Color::Black;
    m += 2;
  }
  //
  return system;
}   // end makeNotenzeile()

// ---------------------------------------------------
int drawSystemWithKey(sf::VertexArray &  system,
                      sf::RenderWindow & window,
                      sf::Sprite &       key)
{
  window.draw(system);
  sf::Rect<float> rect = system.getBounds();
  key.setPosition(10, rect.top - 8);
  window.draw(key);
  return 0;
}   // end drawSystemWithKey()

// ---------------------------------------------------
// draw a note given by its name and octave
// Note: this is not serious - but a beginning
int drawNote(sf::VertexArray  & system,
             sf::RenderWindow & window,
             sf::Sprite       & note,
             const char       * name,  // something like "g1" or "c2"
             int                xp)
{
  struct NotePos
  { 
    char nname;
    int  octave;
    int  yoffs;
  };
  static NotePos notePos[]  // should be completed
  {
    {'g', 1, 25  },
    {'a', 1, 30 },
    {'b', 1, 35 },
    {'c', 2, 40 },
    { 0,  0,  0  }   // end mark
  };
  if (name == NULL)
    return -1;
  if (::strlen(name) < 2)
    return -2;
  // get the right Y offset for the given note
  // simple sequential  search
  int yOffs = 0;
  for (int n=0; ; n++)
  {
    if (notePos[n].nname == 0) break;
    if (notePos[n].nname == name[0])
    {
      if (notePos[n].octave == (name[1] - '0'))
      {
         yOffs = notePos[n].yoffs;
         break;
      }
    }
  }
  sf::Rect<float> rect = system.getBounds();
  note.setPosition(xp, rect.top - yOffs);
  window.draw(note);
  return 0;
}   // end drawSystemWithKey()

// --------------------------------------------------
// Important: the sf::Image, sf::Color and sf::Texture objects 
// used here must be static or allocated on the heap!
int loadSymbol(const char * fileName, 
               sf::Sprite * violKeySp)
{
  sf::Image * violKey = new sf::Image();
  sf::Color * col = new sf::Color(255,255,255);
  sf::Texture * violKeyTx = new sf::Texture();
  //
  bool ret = violKey->loadFromFile(fileName);
  if (! ret)
  {
    std::cout << "file " << fileName << " not found" << std::endl;
    return -1;;
  }
  std::cout << "loadSymbol() image loaded" << std::endl;
  // create a transparency mask
  violKey->createMaskFromColor(*col, 20);
  violKeyTx->loadFromImage(*violKey);
  std::cout << "loadSymbol() texture loaded" << std::endl;
  violKeySp->setTexture(*violKeyTx);
  return 0;   // OK
}   // end loadSymbol()
  
// --------------------------------------------------
int main()
{
  int ret;
  sf::Clock clock;
  sf::Time elapsed;
  int64_t t64;
  sf::Sprite violKeySp, note4;
  unsigned long count = 0L;
  #define WINW 1000
  #define WINH 800  
  //
  // Create the main window
  sf::RenderWindow window(sf::VideoMode(WINW, WINH), "SFML window");
  //
  // load the violin key sprite (a Windows bitmap)
  ret = ::loadSymbol("./keyv2.bmp", &violKeySp);
  // load a sprite for a quarter note
  ret = ::loadSymbol("./note_4.bmp", &note4);
  //
  // Create a graphical text to display
  sf::Font font;
  // EH: path added for font
  #ifdef WIN32
  ret = font.loadFromFile("c:\\windows\\fonts\\arial.ttf");
  #else
  ret = font.loadFromFile("/usr/share/fonts/truetype/dejavu/DejaVuSans-Bold.ttf");
  #endif
  if (! ret)
  {
    std::cout << "font DejaVuSans-Bold.ttf not found" << std::endl;
    return EXIT_FAILURE;
  }
  sf::Text text("SFML Notenbeispiel EH", font, 38);
  text.setPosition(250, 20);
  text.setColor(sf::Color::Black);
  text.setRotation(0.0);
  //
  // rectangles are the replacement for a thick line
  /*
  sf::RectangleShape rectangle;
  rectangle.setSize(sf::Vector2f(100, 80));
  rectangle.setOutlineColor(sf::Color::Red);
  rectangle.setOutlineThickness(5);
  rectangle.setPosition(30, 70);
  */
  //
  // Define five music systems
  sf::VertexArray system1 = makeNotenzeile(10, WINW - 10, (WINH / 6) + 0);
  sf::VertexArray system2 = makeNotenzeile(10, WINW - 10, (WINH / 6) + 80);
  sf::VertexArray system3 = makeNotenzeile(10, WINW - 10, (WINH / 6) + 160);
  sf::VertexArray system4 = makeNotenzeile(10, WINW - 10, (WINH / 6) + 240);
  sf::VertexArray system5 = makeNotenzeile(10, WINW - 10, (WINH / 6) + 320);
  sf::VertexArray system6 = makeNotenzeile(10, WINW - 10, (WINH / 6) + 400);
  sf::VertexArray system7 = makeNotenzeile(10, WINW - 10, (WINH / 6) + 480);
  sf::VertexArray system8 = makeNotenzeile(10, WINW - 10, (WINH / 6) + 560);
  sf::VertexArray system9 = makeNotenzeile(10, WINW - 10, (WINH / 6) + 640);
  //
  float rot = 0.0;
  //
  // Start the loop
  while (window.isOpen())
  {
    // Process events
    sf::Event event;
    while (window.pollEvent(event))
    {
      // Close window: exit
      if (event.type == sf::Event::Closed)
      {
        std::cout << "Exit required" << std::endl;
        window.close();
      }
    }
    //
    clock.restart();
    //
    // Clear screen, set White as background color
    window.clear(sf::Color::White);
    //
    // Draw the string
    window.draw(text);
    //
    //window.draw(rectangle);
    //
    // draw the music systems (5 horizontal lines per system)
    ::drawSystemWithKey(system1, window, violKeySp);
    ::drawSystemWithKey(system2, window, violKeySp);
    ::drawSystemWithKey(system3, window, violKeySp);
    ::drawSystemWithKey(system4, window, violKeySp);
    ::drawSystemWithKey(system5, window, violKeySp);
    ::drawSystemWithKey(system6, window, violKeySp);
    ::drawSystemWithKey(system7, window, violKeySp);
    ::drawSystemWithKey(system8, window, violKeySp);
    ::drawSystemWithKey(system9, window, violKeySp);
    //
    // draw some notes
    int xp = 50;
    for (int n=0; n < 5; n++)
    {
      if ((n % 2) == 0)
        ::drawNote(system1, window, note4, "g1", xp);
      else
        ::drawNote(system1, window, note4, "a1", xp);
      xp += 50;
    }
    xp = 50;
    for (int n=0; n < 5; n++)
    {
      if ((n % 2) == 0)
        ::drawNote(system2, window, note4, "b1", xp);
      else
        ::drawNote(system2, window, note4, "c2", xp);
      xp += 50;
    }
    //
    // Update the window
    window.display();
    //
    // performance measurement
    /*
    elapsed = clock.restart();
    t64 = elapsed.asMicroseconds();
    //std::cout << "count=" << count << " t=" << t64 << std::endl;
    count++;
    */
  }
  return EXIT_SUCCESS;
}   // end main()
